/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor.spi.messages;

import java.util.Locale;
import org.jdtaus.core.text.Message;

/**
 * Message stating that a number of transactions is incompatible with a
 * set of given {@code Textschluessel}.
 */
public final class TransactionCountConstraintMessage extends Message
{
    //--Message-----------------------------------------------------------------

    /**
     * Gets the arguments to use for formatting the message text.
     *
     * @param locale the locale for the argumetns or {@code null} for
     * {@code Locale.getDefault()}.
     *
     * @return the arguments to use for formatting the message text
     * ([{@code transactionCount}, {@code textschluesselCount}]).
     */
    public Object[] getFormatArguments(final Locale locale)
    {
        return new Object[] {
            new Integer(this.getTransactionCount()),
            new Integer(this.getTextschluesselCount())
        };
    }

    /**
     * Gets the text of the message.
     *
     * @return the text of the message.
     */
    public String getText(final Locale locale)
    {
        return TransactionCountConstraintMessageBundle.
            getTransactionCountConstraintMessage(locale).
            format(this.getFormatArguments(locale));

    }

    //-----------------------------------------------------------------Message--
    //--TransactionCountConstraintMessage---------------------------------------

    /**
     * Number of transactions.
     * @serial
     */
    private int transactionCount;

    /**
     * Number of given {@code Textschluessel}.
     * @serial
     */
    private int textschluesselCount;

    /**
     * Creates a new {@code TransactionCountConstraintMessage} taking a
     * number of transactions and a number of {@code Textschluessel}.
     *
     * @param fileCount number of logical files.
     * @param typeCount number of file types.
     */
    public TransactionCountConstraintMessage(final int transactionCount,
        final int textschluesselCount)
    {
        this.transactionCount = transactionCount;
        this.textschluesselCount = textschluesselCount;
    }

    /**
     * Gets the number of transactions.
     *
     * @return number of transactions.
     */
    public int getTransactionCount()
    {
        return this.transactionCount;
    }

    /**
     * Gets the number of {@code Textschluessel}.
     *
     * @return number of {@code Textschluessel}.
     */
    public int getTextschluesselCount()
    {
        return this.textschluesselCount;
    }

    //---------------------------------------TransactionCountConstraintMessage--
}
